@@ -14,6 +14,7 @@ from io import StringIO
 from itertools import chain
 from itertools import product as itertools_product
 from logging import Logger
+from typing import Optional
 from warnings import warn
 
 import numpy as np
@@ -28,15 +29,16 @@ from aesara.compile.function.types import (
 from aesara.compile.mode import Mode, register_mode
 from aesara.compile.ops import OutputGuard, _output_guard
 from aesara.configdefaults import config
-from aesara.graph.basic import Variable, graph_inputs, io_toposort
+from aesara.graph.basic import Variable, io_toposort
 from aesara.graph.destroyhandler import DestroyHandler
 from aesara.graph.features import BadOptimization
-from aesara.graph.fg import InconsistencyError
-from aesara.graph.op import COp, HasInnerGraph, Op
-from aesara.graph.utils import MethodNotDefined
+from aesara.graph.op import HasInnerGraph, Op
+from aesara.graph.utils import InconsistencyError, MethodNotDefined
 from aesara.link.basic import Container, LocalLinker
+from aesara.link.c.op import COp
 from aesara.link.utils import map_storage, raise_with_op
 from aesara.printing import _debugprint
+from aesara.tensor import TensorType
 from aesara.utils import NoDuplicateOptWarningFilter, difference, get_unbound_function
 
 
@@ -678,7 +680,7 @@ def _lessbroken_deepcopy(a):
     """
     # this exists because copy.deepcopy on numpy arrays is broken
     # This logic is also in link.py
-    from aesara.graph.type import _cdata_type
+    from aesara.link.c.type import _cdata_type
 
     if isinstance(a, (np.ndarray, np.memmap)):
         rval = a.copy(order="K")
@@ -696,8 +698,9 @@ def _lessbroken_deepcopy(a):
 
 
 def _find_bad_optimizations(order, reasons, r_vals):
-    """
-    Use a simple algorithm to find broken optimizations.
+    """Iterate over variables looking for values that don't match the values of the variables they replaced.
+
+    This is a sign of a broken optimization.
 
     This algorithm is simple to understand, but sometimes when there's
     a problem it identifies the wrong optimization as the culprit.
@@ -706,9 +709,6 @@ def _find_bad_optimizations(order, reasons, r_vals):
     graph).
 
     """
-    # iterate over variables looking for values that don't match the
-    # values of the variables they replaced.  This is the sign of a
-    # broken optimization.
     for i, node in enumerate(order):
         for new_r in node.outputs:
             for reason, r, old_graph_str, new_graph_str in reasons[new_r]:
@@ -753,16 +753,6 @@ def _get_preallocated_maps(
     Preallocate outputs in different memory layouts.
 
     """
-
-    # To avoid circular imports
-    from aesara.gpuarray import GpuArrayType
-    from aesara.tensor.type import TensorType
-
-    try:
-        import pygpu
-    except ImportError:
-        pass
-
     # TODO: Sparse? Scalar does not really make sense.
 
     # Do not preallocate memory for outputs that actually work inplace
@@ -800,7 +790,7 @@ def _get_preallocated_maps(
         for r in considered_outputs:
             # There is no risk to overwrite inputs, since r does not work
             # inplace.
-            if isinstance(r.type, (TensorType, GpuArrayType)):
+            if isinstance(r.type, TensorType):
                 reuse_outputs[r][...] = np.asarray(def_val).astype(r.type.dtype)
 
         if reuse_outputs:
@@ -813,7 +803,7 @@ def _get_preallocated_maps(
     if "c_contiguous" in prealloc_modes or "ALL" in prealloc_modes:
         c_cont_outputs = {}
         for r in considered_outputs:
-            if isinstance(r.type, (TensorType, GpuArrayType)):
+            if isinstance(r.type, TensorType):
                 # Build a C-contiguous buffer
                 new_buf = r.type.value_zeros(r_vals[r].shape)
                 assert new_buf.flags["C_CONTIGUOUS"]
@@ -830,13 +820,11 @@ def _get_preallocated_maps(
     if "f_contiguous" in prealloc_modes or "ALL" in prealloc_modes:
         f_cont_outputs = {}
         for r in considered_outputs:
-            if isinstance(r.type, (TensorType, GpuArrayType)):
+            if isinstance(r.type, TensorType):
                 new_buf = np.zeros(
                     shape=r_vals[r].shape, dtype=r_vals[r].dtype, order="F"
                 )
                 new_buf[...] = def_val
-                if isinstance(r.type, GpuArrayType):
-                    new_buf = pygpu.array(new_buf)
 
                 f_cont_outputs[r] = new_buf
 
@@ -860,7 +848,7 @@ def _get_preallocated_maps(
         max_ndim = 0
         rev_out_broadcastable = []
         for r in considered_outputs:
-            if isinstance(r.type, (TensorType, GpuArrayType)):
+            if isinstance(r.type, TensorType):
                 if max_ndim < r.ndim:
                     rev_out_broadcastable += [True] * (r.ndim - max_ndim)
                     max_ndim = r.ndim
@@ -875,7 +863,7 @@ def _get_preallocated_maps(
         # Initial allocation
         init_strided = {}
         for r in considered_outputs:
-            if isinstance(r.type, (TensorType, GpuArrayType)):
+            if isinstance(r.type, TensorType):
                 # Create a buffer twice as large in every dimension,
                 # except if broadcastable, or for dimensions above
                 # config.DebugMode__check_preallocated_output_ndim
@@ -954,7 +942,7 @@ def _get_preallocated_maps(
                 name = f"wrong_size{tuple(shape_diff)}"
 
                 for r in considered_outputs:
-                    if isinstance(r.type, (TensorType, GpuArrayType)):
+                    if isinstance(r.type, TensorType):
                         r_shape_diff = shape_diff[: r.ndim]
                         out_shape = [
                             max((s + sd), 0)
@@ -1365,14 +1353,14 @@ class _Linker(LocalLinker):
         self.maker = maker
         super().__init__(scheduler=schedule)
 
-    def accept(self, fgraph, no_recycling=None, profile=None):
+    def accept(self, fgraph, no_recycling: Optional[list] = None, profile=None):
         if no_recycling is None:
             no_recycling = []
         if self.fgraph is not None and self.fgraph is not fgraph:
             assert type(self) is _Linker
             return type(self)(maker=self.maker).accept(fgraph, no_recycling, profile)
         self.fgraph = fgraph
-        self.no_recycling = no_recycling
+        self.no_recycling: list = no_recycling
         return self
 
     def make_all(
@@ -1381,7 +1369,6 @@ class _Linker(LocalLinker):
         # can't import at toplevel because of circular import TODO:
         # don't do this ugly hacky way of setting the
         # filter_checks_isfinite
-        from aesara.tensor.type import TensorType  # to set filter_check_isfinite
 
         fgraph = self.fgraph
         input_storage_ = input_storage
@@ -1404,7 +1391,7 @@ class _Linker(LocalLinker):
         # check_preallocated_output even on the output of the function.
         # no_recycling in individual thunks does not really matter, since
         # the function's outputs will always be freshly allocated.
-        no_recycling = []
+        no_recycling: list = []
 
         input_storage, output_storage, storage_map = map_storage(
             fgraph, order, input_storage_, output_storage_, storage_map
@@ -1495,6 +1482,7 @@ class _Linker(LocalLinker):
         # Use self.no_recycling (that was passed in accept()) to always
         # use new memory storage when it is needed, in particular for the
         # function's outputs. no_recycling_map will be used in f() below.
+        no_recycling_map: list = []
         if self.no_recycling is True:
             no_recycling_map = list(storage_map.values())
             no_recycling_map = difference(no_recycling_map, input_storage)
@@ -2041,20 +2029,10 @@ class _Maker(FunctionMaker):  # inheritance buys a few helper functions
         inputs = [self.wrap_in(i) for i in inputs]
         outputs = [self.wrap_out(o) for o in outputs]
 
-        _inputs = list(
-            graph_inputs(
-                [o.variable for o in outputs]
-                + [i.update for i in inputs if getattr(i, "update", False)]
-            )
-        )
-
         # Check if some input variables are unused
-        self._check_unused_inputs(inputs, outputs, on_unused_input)
+        self.check_unused_inputs(inputs, outputs, on_unused_input)
 
-        # Make a list of (SymbolicInput|SymblicInputKits, indices,
-        # [SymbolicInput,...]), one tuple for each input. (See
-        # Function.indices for more details)
-        indices = [[input] + self.expand_in(input, _inputs) for input in inputs]
+        indices = [[input, None, [input]] for input in inputs]
 
         # make the fgraph
         for i in range(mode.stability_patience):
