@@ -1572,7 +1572,7 @@ class ProfileStats:
 
         # tip 2
         if not config.lib__amblibm and any(
-            [amdlibm_speed_up(a.op) for (fgraph, a) in self.apply_time]
+            amdlibm_speed_up(a.op) for (fgraph, a) in self.apply_time
         ):
             print(
                 "  - Try installing amdlibm and set the Aesara flag "
@@ -1584,10 +1584,8 @@ class ProfileStats:
 
         # tip 3
         if not config.lib__amblibm and any(
-            [
-                exp_float32_op(a.op) and a.inputs[0].dtype == "float32"
-                for (fgraph, a) in self.apply_time
-            ]
+            exp_float32_op(a.op) and a.inputs[0].dtype == "float32"
+            for (fgraph, a) in self.apply_time
         ):
             print(
                 "  - With the default gcc libm, exp in float32 is slower "
@@ -1601,7 +1599,7 @@ class ProfileStats:
         for (fgraph, a) in self.apply_time:
             node = a
             if isinstance(node.op, Dot) and all(
-                [len(i.type.broadcastable) == 2 for i in node.inputs]
+                len(i.type.broadcastable) == 2 for i in node.inputs
             ):
                 print(
                     (
