@@ -2,7 +2,6 @@ import logging
 import os
 import shlex
 import sys
-import typing
 import warnings
 from configparser import (
     ConfigParser,
@@ -13,7 +12,7 @@ from configparser import (
 )
 from functools import wraps
 from io import StringIO
-from typing import Optional
+from typing import Callable, Dict, Optional, Sequence, Union
 
 from aesara.utils import deprecated, hash_from_code
 
@@ -22,14 +21,13 @@ _logger = logging.getLogger("aesara.configparser")
 
 
 class AesaraConfigWarning(Warning):
+    @classmethod
     def warn(cls, message, stacklevel=0):
         warnings.warn(message, cls, stacklevel=stacklevel + 3)
 
-    warn = classmethod(warn)
-
 
 class ConfigAccessViolation(AttributeError):
-    """ Raised when a config setting is accessed through the wrong config instance. """
+    """Raised when a config setting is accessed through the wrong config instance."""
 
 
 class _ChangeFlagsDecorator:
@@ -89,13 +87,13 @@ class _SectionRedirect:
 
 
 class AesaraConfigParser:
-    """ Object that holds configuration settings. """
+    """Object that holds configuration settings."""
 
     def __init__(self, flags_dict: dict, aesara_cfg, aesara_raw_cfg):
         self._flags_dict = flags_dict
         self._aesara_cfg = aesara_cfg
         self._aesara_raw_cfg = aesara_raw_cfg
-        self._config_var_dict: typing.Dict = {}
+        self._config_var_dict: Dict = {}
         super().__init__()
 
     def __str__(self, print_doc=True):
@@ -269,10 +267,10 @@ class ConfigParam:
 
     def __init__(
         self,
-        default: typing.Union[object, typing.Callable[[object], object]],
+        default: Union[object, Callable[[object], object]],
         *,
-        apply: typing.Optional[typing.Callable[[object], object]] = None,
-        validate: typing.Optional[typing.Callable[[object], bool]] = None,
+        apply: Optional[Callable[[object], object]] = None,
+        validate: Optional[Callable[[object], bool]] = None,
         mutable: bool = True,
     ):
         """
@@ -374,7 +372,7 @@ class ConfigParam:
 
 class EnumStr(ConfigParam):
     def __init__(
-        self, default: str, options: typing.Sequence[str], validate=None, mutable=True
+        self, default: str, options: Sequence[str], validate=None, mutable=True
     ):
         """Creates a str-based parameter that takes a predefined set of options.
 
@@ -458,24 +456,22 @@ class DeviceParam(ConfigParam):
         )
 
     def _apply(self, val):
-        if val == self.default or val.startswith("opencl") or val.startswith("cuda"):
-            return val
-        elif val.startswith("gpu"):
+        if val.startswith("opencl") or val.startswith("cuda") or val.startswith("gpu"):
             raise ValueError(
                 "You are trying to use the old GPU back-end. "
-                "It was removed from Aesara. Use device=cuda* now. "
-                "See https://github.com/aesara-devs/aesara/wiki/Converting-to-the-new-gpu-back-end%28gpuarray%29 "
-                "for more information."
+                "It was removed from Aesara."
             )
+        elif val == self.default:
+            return val
         else:
             raise ValueError(
                 'Invalid value ("{val}") for configuration '
                 'variable "{self.name}". Valid options start with '
-                'one of "cpu", "opencl" or "cuda".'
+                'one of "cpu".'
             )
 
     def __str__(self):
-        return f"{self.name} ({self.default}, opencl*, cuda*) "
+        return f"{self.name} ({self.default})"
 
 
 class ContextsParam(ConfigParam):
