@@ -1,49 +1,41 @@
 """A container for specifying and manipulating a graph with distinct inputs and outputs."""
 import time
 from collections import OrderedDict
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import (
+    Any,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    Union,
+    cast,
+)
+
+from typing_extensions import Literal
 
 import aesara
 from aesara.configdefaults import config
-from aesara.graph.basic import Apply, Constant, Variable, applys_between
+from aesara.graph.basic import Apply, Constant, Node, Variable, applys_between
 from aesara.graph.basic import as_string as graph_as_string
 from aesara.graph.basic import clone_get_equiv, graph_inputs, io_toposort, vars_between
 from aesara.graph.features import AlreadyThere, Feature, ReplaceValidate
-from aesara.graph.utils import MetaObject, TestValueError, get_variable_trace_string
+from aesara.graph.utils import MetaObject, MissingInputError, TestValueError
 from aesara.misc.ordered_set import OrderedSet
 
 
-class InconsistencyError(Exception):
-    """
-    This exception should be thrown by listeners to FunctionGraph when the
-    graph's state is invalid.
-
-    """
-
-
-class MissingInputError(Exception):
-    """
-    A symbolic input needed to compute the outputs is missing.
-
-    """
-
-    def __init__(self, *args, **kwargs):
-        if kwargs:
-            # The call to list is needed for Python 3
-            assert list(kwargs.keys()) == ["variable"]
-            error_msg = get_variable_trace_string(kwargs["variable"])
-            if error_msg:
-                args = args + (error_msg,)
-        s = "\n".join(args)  # Needed to have the new line print correctly
-        super().__init__(s)
+ApplyOrOutput = Union[Apply, Literal["output"]]
+ClientType = Tuple[ApplyOrOutput, int]
 
 
 class FunctionGraph(MetaObject):
-    """
+    r"""
     A `FunctionGraph` represents a subgraph bound by a set of input variables and
     a set of output variables, ie a subgraph that specifies an Aesara function.
     The inputs list should contain all the inputs on which the outputs depend.
-    ``Variable``s of type ``Constant`` are not counted as inputs.
+    `Variable`\s of type `Constant` are not counted as inputs.
 
     The `FunctionGraph` supports the replace operation which allows to replace
     a variable in the subgraph by another, e.g. replace ``(x + x).out`` by
@@ -51,34 +43,30 @@ class FunctionGraph(MetaObject):
 
     This class is also responsible for verifying that a graph is valid
     (ie, all the dtypes and broadcast patterns are compatible with the
-    way the ``Variable``s are used) and for tracking the ``Variable``s with
-    a ``clients`` field that specifies which ``Apply`` nodes use the ``Variable``.
-    The ``clients`` field combined with the ``Variable.owner`` field and the
-    ``Apply`` nodes' ``Apply.inputs`` field allows the graph to be traversed in
-    both directions.
+    way the `Variable`\s are used) and for tracking the `Variable`\s with
+    a :attr:`FunctionGraph.clients` ``dict`` that specifies which `Apply` nodes
+    use the `Variable`.  The :attr:`FunctionGraph.clients` field, combined with
+    the :attr:`Variable.owner` and each :attr:`Apply.inputs`, allows the graph
+    to be traversed in both directions.
 
     It can also be extended with new features using
-    ``FunctionGraph.attach_feature(<Feature instance>)``.
-    See ``Feature`` for event types and documentation.
-    Extra features allow the `FunctionGraph` to verify new properties of
-    a graph as it is optimized.
-
-    Historically, the `FunctionGraph` was called an ``Env``. Keep this in mind
-    while reading out-of-date documentation, e-mail support threads, etc.
+    :meth:`FunctionGraph.attach_feature`.  See `Feature` for event types and
+    documentation.  Extra features allow the `FunctionGraph` to verify new
+    properties of a graph as it is optimized.
 
     The constructor creates a `FunctionGraph` which operates on the subgraph
     bound by the inputs and outputs sets.
 
-    This class keeps a pointer to the inputs and outputs, and also modifies
-    them.
+    This class keeps lists for the inputs and outputs and modifies them
+    in-place.
 
     """
 
     def __init__(
         self,
-        inputs: Optional[List[Variable]] = None,
-        outputs: Optional[List[Variable]] = None,
-        features: Optional[List[Feature]] = None,
+        inputs: Optional[Sequence[Variable]] = None,
+        outputs: Optional[Sequence[Variable]] = None,
+        features: Optional[Sequence[Feature]] = None,
         clone: bool = True,
         update_mapping: Optional[Dict[Variable, Variable]] = None,
         memo: Optional[Dict[Variable, Variable]] = None,
@@ -103,11 +91,11 @@ class FunctionGraph(MetaObject):
             Mapping between the `inputs` with updates and the `outputs`
             corresponding to their updates.
         memo
-            See ``clone_get_equiv``.
+            See :func:`aesara.graph.basic.clone_get_equiv`.
         copy_inputs
-            See ``clone_get_equiv``.
+            See :func:`aesara.graph.basic.clone_get_equiv`.
         copy_orphans
-            See ``clone_get_equiv``.
+            See :func:`aesara.graph.basic.clone_get_equiv`.
         """
         if outputs is None:
             raise ValueError("No outputs specified")
@@ -116,36 +104,36 @@ class FunctionGraph(MetaObject):
             inputs = [i for i in graph_inputs(outputs) if not isinstance(i, Constant)]
 
         if clone:
-            memo = clone_get_equiv(
+            _memo = clone_get_equiv(
                 inputs,
                 outputs,
                 copy_inputs=copy_inputs,
                 copy_orphans=copy_orphans,
-                memo=memo,
+                memo=cast(Dict[Node, Node], memo),
             )
-            outputs = [memo[o] for o in outputs]
-            inputs = [memo[i] for i in inputs]
+            outputs = [cast(Variable, _memo[o]) for o in outputs]
+            inputs = [cast(Variable, _memo[i]) for i in inputs]
 
-        self.execute_callbacks_time = 0
-        self.execute_callbacks_times = {}
+        self.execute_callbacks_time: float = 0.0
+        self.execute_callbacks_times: Dict[Feature, float] = {}
 
         if features is None:
             features = []
 
-        self._features = []
+        self._features: List[Feature] = []
 
         # All apply nodes in the subgraph defined by inputs and
         # outputs are cached in this field
-        self.apply_nodes = set()
+        self.apply_nodes: Set[Apply] = set()
 
         # Ditto for variable nodes.
         # It must contain all fgraph.inputs and all apply_nodes
         # outputs even if they aren't used in the graph.
-        self.variables = set()
+        self.variables: Set[Variable] = set()
 
-        self.inputs = []
-        self.outputs = list(outputs)
-        self.clients = {}
+        self.inputs: List[Variable] = []
+        self.outputs: List[Variable] = list(outputs)
+        self.clients: Dict[Variable, List[ClientType]] = {}
 
         for f in features:
             self.attach_feature(f)
@@ -196,47 +184,11 @@ class FunctionGraph(MetaObject):
         """
         self.clients.setdefault(var, [])
 
-    def setup_node(self, node: Apply) -> None:
-        """Set up node so it belongs to this `FunctionGraph`.
-
-        Parameters
-        ----------
-        node : aesara.graph.basic.Apply
-
-        """
-        if node.op.view_map and not all(
-            isinstance(view, (list, tuple)) for view in node.op.view_map.values()
-        ):
-            raise Exception(
-                f"Op '{node.op}' have a bad view map '{node.op.view_map}',"
-                " the values must be tuples or lists."
-            )
-        if node.op.destroy_map and not all(
-            isinstance(destroy, (list, tuple))
-            for destroy in node.op.destroy_map.values()
-        ):
-            raise Exception(
-                f"Op '{node.op}' have a bad destroy map '{node.op.destroy_map}',"
-                " the values must be tuples or lists."
-            )
-
-    def disown(self) -> None:
-        """Clear internal variables."""
-        for f in self._features:
-            self.remove_feature(f)
-        self.clients = {}
-        self.apply_nodes = set()
-        self.variables = set()
-        self.inputs = None
-        self.outputs = None
-        self.profile = None
-        self.update_mapping = None
-
-    def get_clients(self, var: Variable) -> List[Tuple[Apply, int]]:
+    def get_clients(self, var: Variable) -> List[ClientType]:
         """Return a list of all the `(node, i)` pairs such that `node.inputs[i]` is `var`."""
         return self.clients[var]
 
-    def add_client(self, var: Variable, new_client: Tuple[Apply, int]) -> None:
+    def add_client(self, var: Variable, new_client: ClientType) -> None:
         """Update the clients of `var` with `new_clients`.
 
         Parameters
@@ -247,10 +199,17 @@ class FunctionGraph(MetaObject):
             A ``(node, i)`` pair such that ``node.inputs[i]`` is `var`.
 
         """
+        if not isinstance(new_client[0], Apply) and new_client[0] != "output":
+            raise TypeError(
+                'The first entry of `new_client` must be an `Apply` node or the string `"output"`'
+            )
         self.clients[var].append(new_client)
 
     def remove_client(
-        self, var: Variable, client_to_remove: Tuple[Apply, int], reason: str = None
+        self,
+        var: Variable,
+        client_to_remove: ClientType,
+        reason: Optional[str] = None,
     ) -> None:
         """Recursively remove clients of a variable.
 
@@ -316,7 +275,7 @@ class FunctionGraph(MetaObject):
                         removal_stack.append((in_var, (apply_node, i)))
 
     def import_var(
-        self, var: Variable, reason: str = None, import_missing: bool = False
+        self, var: Variable, reason: Optional[str] = None, import_missing: bool = False
     ) -> None:
         """Import variables into this `FunctionGraph`.
 
@@ -357,7 +316,7 @@ class FunctionGraph(MetaObject):
         self,
         apply_node: Apply,
         check: bool = True,
-        reason: str = None,
+        reason: Optional[str] = None,
         import_missing: bool = False,
     ) -> None:
         """Recursively import everything between an ``Apply`` node and the ``FunctionGraph``'s outputs.
@@ -403,7 +362,6 @@ class FunctionGraph(MetaObject):
 
         for node in new_nodes:
             assert node not in self.apply_nodes
-            self.setup_node(node)
             self.apply_nodes.add(node)
             if not hasattr(node.tag, "imported_by"):
                 node.tag.imported_by = []
@@ -420,10 +378,10 @@ class FunctionGraph(MetaObject):
 
     def change_node_input(
         self,
-        node: Union[Apply, str],
+        node: ApplyOrOutput,
         i: int,
         new_var: Variable,
-        reason: str = None,
+        reason: Optional[str] = None,
         import_missing: bool = False,
         check: bool = True,
     ) -> None:
@@ -464,6 +422,7 @@ class FunctionGraph(MetaObject):
                 )
             self.outputs[i] = new_var
         else:
+            assert isinstance(node, Apply)
             r = node.inputs[i]
             if check and not r.type.is_super(new_var.type):
                 raise TypeError(
@@ -487,8 +446,8 @@ class FunctionGraph(MetaObject):
         self,
         var: Variable,
         new_var: Variable,
-        reason: str = None,
-        verbose: bool = None,
+        reason: Optional[str] = None,
+        verbose: Optional[bool] = None,
         import_missing: bool = False,
     ) -> None:
         """Replace a variable in the `FunctionGraph`.
@@ -513,7 +472,9 @@ class FunctionGraph(MetaObject):
         if verbose is None:
             verbose = config.optimizer_verbose
         if verbose:
-            print(f"optimizer: rewrite {reason} replaces {var} with {new_var}")
+            print(
+                f"optimizer: rewrite {reason} replaces {var} of {var.owner} with {new_var} of {new_var.owner}"
+            )
 
         new_var = var.type.filter_variable(new_var, allow_convert=True)
 
@@ -545,13 +506,13 @@ class FunctionGraph(MetaObject):
 
         for node, i in list(self.clients[var]):
             assert (node == "output" and self.outputs[i] is var) or (
-                node.inputs[i] is var
+                isinstance(node, Apply) and node.inputs[i] is var
             )
             self.change_node_input(
                 node, i, new_var, reason=reason, import_missing=import_missing
             )
 
-    def replace_all(self, pairs: List[Tuple[Variable, Variable]], **kwargs) -> None:
+    def replace_all(self, pairs: Iterable[Tuple[Variable, Variable]], **kwargs) -> None:
         """Replace variables in the `FunctionGraph` according to ``(var, new_var)`` pairs in a list."""
         for var, new_var in pairs:
             self.replace(var, new_var, **kwargs)
@@ -572,7 +533,7 @@ class FunctionGraph(MetaObject):
                 attach(self)
             except AlreadyThere:
                 return
-        self.execute_callbacks_times.setdefault(feature, 0)
+        self.execute_callbacks_times.setdefault(feature, 0.0)
         # It would be nice if we could require a specific class instead of
         # a "workalike" so we could do actual error checking
         # if not isinstance(feature, Feature):
@@ -635,48 +596,40 @@ class FunctionGraph(MetaObject):
         return d
 
     def toposort(self) -> List[Apply]:
-        """Return a toposorted list of the nodes.
+        r"""Return a toposorted list of the nodes.
 
-        Return an ordering of the graph's ``Apply`` nodes such that:
+        Return an ordering of the graph's :class:`Apply` nodes such that:
 
-        * all the nodes of the inputs of a node are before that node and
-        * they satisfy the orderings provided by each feature that has
-          an ``orderings`` method.
+        * all the nodes of the inputs of a node are before that node, and
+        * they satisfy the additional orderings provided by
+          :meth:`FunctionGraph.orderings`.
 
-        If a feature has an ``orderings`` method, it will be called with
-        this `FunctionGraph` as sole argument. It should return a dictionary of
-        ``{node: predecessors}`` where predecessors is a list of nodes that
-        should be computed before the key node.
         """
         if len(self.apply_nodes) < 2:
-            # optimization
-            # when there are 0 or 1 nodes, no sorting is necessary
-            # This special case happens a lot because the OpWiseCLinker
-            # produces 1-element graphs.
+            # No sorting is necessary
             return list(self.apply_nodes)
-        fg = self
 
-        ords = self.orderings()
-
-        order = io_toposort(fg.inputs, fg.outputs, ords)
-
-        return order
+        return io_toposort(self.inputs, self.outputs, self.orderings())
 
     def orderings(self) -> Dict[Apply, List[Apply]]:
-        """Return ``dict`` ``d`` s.t. ``d[node]`` is a list of nodes that must be evaluated before ``node`` itself can be evaluated.
+        """Return a map of node to node evaluation dependencies.
 
-        This is used primarily by the ``destroy_handler`` feature to ensure that
-        the clients of any destroyed inputs have already computed their
-        outputs.
+        Each key node is mapped to a list of nodes that must be evaluated
+        before the key nodes can be evaluated.
+
+        This is used primarily by the :class:`DestroyHandler` :class:`Feature`
+        to ensure that the clients of any destroyed inputs have already
+        computed their outputs.
 
         Notes
         -----
-        This only calls the ``orderings()`` function on all features. It does not
+        This only calls the :meth:`Feature.orderings` method of each
+        :class:`Feature` attached to the :class:`FunctionGraph`. It does not
         take care of computing the dependencies by itself.
 
         """
         assert isinstance(self._features, list)
-        all_orderings = []
+        all_orderings: List[OrderedDict] = []
 
         for feature in self._features:
             if hasattr(feature, "orderings"):
@@ -702,7 +655,7 @@ class FunctionGraph(MetaObject):
             return all_orderings[0].copy()
         else:
             # If there is more than 1 ordering, combine them.
-            ords = OrderedDict()
+            ords: Dict[Apply, List[Apply]] = OrderedDict()
             for orderings in all_orderings:
                 for node, prereqs in orderings.items():
                     ords.setdefault(node, []).extend(prereqs)
@@ -712,12 +665,12 @@ class FunctionGraph(MetaObject):
         """Check the integrity of nodes in the graph."""
         nodes = set(applys_between(self.inputs, self.outputs))
         if self.apply_nodes != nodes:
-            missing = nodes.difference(self.apply_nodes)
-            excess = self.apply_nodes.difference(nodes)
+            nodes_missing = nodes.difference(self.apply_nodes)
+            nodes_excess = self.apply_nodes.difference(nodes)
             raise Exception(
                 "The nodes are inappropriately cached. missing, in excess: ",
-                missing,
-                excess,
+                nodes_missing,
+                nodes_excess,
             )
         for node in nodes:
             for i, variable in enumerate(node.inputs):
@@ -728,12 +681,12 @@ class FunctionGraph(MetaObject):
                     )
         variables = set(vars_between(self.inputs, self.outputs))
         if set(self.variables) != variables:
-            missing = variables.difference(self.variables)
-            excess = self.variables.difference(variables)
+            vars_missing = variables.difference(self.variables)
+            vars_excess = self.variables.difference(variables)
             raise Exception(
                 "The variables are inappropriately cached. missing, in excess: ",
-                missing,
-                excess,
+                vars_missing,
+                vars_excess,
             )
         for variable in variables:
             if (
@@ -742,20 +695,23 @@ class FunctionGraph(MetaObject):
                 and not isinstance(variable, Constant)
             ):
                 raise Exception(f"Undeclared input: {variable}")
-            for node, i in self.clients[variable]:
-                if node == "output":
+            for cl_node, i in self.clients[variable]:
+                if cl_node == "output":
                     if self.outputs[i] is not variable:
                         raise Exception(
                             f"Inconsistent clients list: {variable}, {self.outputs[i]}"
                         )
                     continue
-                if node not in nodes:
+
+                assert isinstance(cl_node, Apply)
+
+                if cl_node not in nodes:
                     raise Exception(
-                        f"Client not in FunctionGraph: {variable}, {(node, i)}"
+                        f"Client not in FunctionGraph: {variable}, {(cl_node, i)}"
                     )
-                if node.inputs[i] is not variable:
+                if cl_node.inputs[i] is not variable:
                     raise Exception(
-                        f"Inconsistent clients list: {variable}, {node.inputs[i]}"
+                        f"Inconsistent clients list: {variable}, {cl_node.inputs[i]}"
                     )
 
     def __repr__(self):
@@ -767,7 +723,7 @@ class FunctionGraph(MetaObject):
 
     def clone_get_equiv(
         self, check_integrity: bool = True, attach_feature: bool = True
-    ) -> Union["FunctionGraph", Dict[Variable, Variable]]:
+    ) -> Tuple["FunctionGraph", Dict[Node, Node]]:
         """Clone the graph and return a ``dict`` that maps old nodes to new nodes.
 
         Parameters
@@ -789,8 +745,8 @@ class FunctionGraph(MetaObject):
         if check_integrity:
             self.check_integrity()
         e = FunctionGraph(
-            [equiv[i] for i in self.inputs],
-            [equiv[o] for o in self.outputs],
+            [cast(Variable, equiv[i]) for i in self.inputs],
+            [cast(Variable, equiv[o]) for o in self.outputs],
             clone=False,
         )
         if check_integrity:
