@@ -1,27 +1,39 @@
 from abc import ABC, abstractmethod
 from copy import copy, deepcopy
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Set, Tuple, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
 
 from numpy import ndarray
 
 from aesara.configdefaults import config
 from aesara.graph.basic import Apply, Variable
 from aesara.graph.fg import FunctionGraph
-from aesara.graph.type import CType
-from aesara.graph.utils import MetaObject
+from aesara.graph.type import Type
 from aesara.link.utils import gc_helper, map_storage, raise_with_op, streamline
-from aesara.utils import deprecated, difference, to_return_values
+from aesara.utils import difference
 
 
 if TYPE_CHECKING:
-
-    from aesara.link.c.basic import OpWiseCLinker
-    from aesara.link.vm import VMLinker
+    from aesara.compile.profiling import ProfileStats
+    from aesara.graph.op import (
+        BasicThunkType,
+        InputStorageType,
+        OutputStorageType,
+        StorageMapType,
+    )
     from aesara.tensor.var import TensorVariable
 
-StorageMapType = Dict[Variable, List[Optional[Union[ndarray, slice]]]]
-OutputStorageType = List[Optional[List[Any]]]
-ThunkType = Tuple[Callable[[], None], List["Container"], List["Container"]]
+
+ThunkAndContainersType = Tuple["BasicThunkType", List["Container"], List["Container"]]
 
 
 class Container:
@@ -32,7 +44,8 @@ class Container:
 
     Parameters
     ----------
-    r : :py:class:`aerasa.graph.utils.MetaObject`
+    r
+        The `Variable` or `Type` to associate with the `Container`.
     storage
         A list of length 1, whose element is the value for `r`.
     readonly : bool
@@ -50,8 +63,8 @@ class Container:
 
     def __init__(
         self,
-        r: MetaObject,
-        storage: Any,
+        r: Union[Variable, Type],
+        storage: List[Any],
         *,
         readonly: bool = False,
         strict: bool = False,
@@ -60,10 +73,11 @@ class Container:
     ) -> None:
         if not isinstance(storage, list) or not len(storage) >= 1:
             raise TypeError("storage must be a list of length at least one")
-        if isinstance(r, CType):
-            self.type = r
-        else:
+        if isinstance(r, Variable):
             self.type = r.type
+        else:
+            self.type = r
+
         if name is None:
             # Some Type do not have a name field.
             self.name = getattr(r, "name", None)
@@ -156,7 +170,7 @@ class Linker(ABC):
         self,
         *,
         allow_gc: Optional[bool] = None,
-        scheduler: Callable[[FunctionGraph], List[Apply]] = None,
+        scheduler: Optional[Callable[[FunctionGraph], List[Apply]]] = None,
     ) -> None:
         self._allow_gc = allow_gc
         self._scheduler = scheduler
@@ -180,7 +194,9 @@ class Linker(ABC):
         return new
 
     @abstractmethod
-    def make_thunk(self, **kwargs) -> ThunkType:
+    def make_thunk(
+        self, **kwargs
+    ) -> Tuple[Callable, "InputStorageType", "OutputStorageType"]:
         """
         This function must return a triplet (function, input_variables,
         output_variables) where function is a thunk that operates on the
@@ -203,50 +219,6 @@ class Linker(ABC):
 
         """
 
-    @deprecated("Marked for deletion. Only tests use it.")
-    def make_function(self, unpack_single: bool = True, **kwargs) -> Callable:
-        """
-        Returns a function that takes values corresponding to the inputs of the
-        fgraph used by this L{Linker} and returns values corresponding the the
-        outputs of that fgraph. If inplace is True, the calculations will
-        operate in the same storage the fgraph uses, else independent storage
-        will be allocated for the function.
-
-        Parameters
-        ----------
-        unpack_single : bool
-            If `unpack_single` is True (default) and that the function has only one
-            output, then that output will be returned. Else, a list or tuple of
-            length 1 will be returned.
-
-        Examples
-        --------
-        e = x + y
-        fgraph = FunctionGraph([x, y], [e])
-        fn = MyLinker(fgraph).make_function(inplace)
-        print fn(1.0, 2.0) # 3.0
-        print e.data # 3.0 iff inplace == True (else unknown)
-
-        """
-        thunk, inputs, outputs = self.make_thunk(**kwargs)
-
-        def execute(*args):
-            takes = len(inputs)
-            got = len(args)
-            if got != takes:
-                raise TypeError(
-                    f"Function call takes exactly {takes} args ({got} given)"
-                )
-            for arg, variable in zip(args, inputs):
-                variable.data = arg
-            thunk()
-            if unpack_single:
-                return to_return_values([variable.data for variable in outputs])
-            else:
-                return [variable.data for variable in outputs]
-
-        return execute
-
     def schedule(self, fgraph: FunctionGraph) -> List[Apply]:
         """Runs the scheduler (if set) or the toposort on the FunctionGraph.
 
@@ -274,26 +246,37 @@ class LocalLinker(Linker):
 
     def make_thunk(
         self,
-        input_storage: Optional[Any] = None,
-        output_storage: Optional[OutputStorageType] = None,
-        storage_map: Optional[StorageMapType] = None,
+        input_storage: Optional["InputStorageType"] = None,
+        output_storage: Optional["OutputStorageType"] = None,
+        storage_map: Optional["StorageMapType"] = None,
         **kwargs,
-    ) -> ThunkType:
+    ) -> Tuple["BasicThunkType", "InputStorageType", "OutputStorageType"]:
         return self.make_all(
             input_storage=input_storage,
             output_storage=output_storage,
             storage_map=storage_map,
         )[:3]
 
-    def make_all(self, input_storage, output_storage, storage_map):
-        # By convention, subclasses of LocalLinker should implement this function!
-        #
-        # This function should return a tuple of 5 things
-        # 1. function to run the program
-        # 2. input storage
-        # 3. output storage
-        # 4. thunks: list of nodes' functions in the order they will be run by the function in (1)
-        # 5. order: list of nodes, in the order they will be run by the function in (1)
+    def make_all(
+        self,
+        input_storage: Optional["InputStorageType"] = None,
+        output_storage: Optional["OutputStorageType"] = None,
+        storage_map: Optional["StorageMapType"] = None,
+    ) -> Tuple[
+        "BasicThunkType",
+        "InputStorageType",
+        "OutputStorageType",
+        List[ThunkAndContainersType],
+        List[Apply],
+    ]:
+        """
+        This function should return a tuple of 5 things
+            1. function to run the program
+            2. input storage
+            3. output storage
+            4. thunks: list of nodes' functions in the order they will be run by the function in (1)
+            5. order: list of nodes, in the order they will be run by the function in (1)
+        """
         raise NotImplementedError(
             f"make_all method of {type(self)} is not implemented."
         )
@@ -306,7 +289,9 @@ class PerformLinker(LocalLinker):
 
     """
 
-    def __init__(self, allow_gc: Optional[bool] = None, schedule: None = None) -> None:
+    def __init__(
+        self, allow_gc: Optional[bool] = None, schedule: Optional[Callable] = None
+    ) -> None:
         if allow_gc is None:
             allow_gc = config.allow_gc
         self.fgraph: Optional[FunctionGraph] = None
@@ -315,26 +300,19 @@ class PerformLinker(LocalLinker):
     def accept(
         self,
         fgraph: FunctionGraph,
-        no_recycling: Optional[
-            Union[List, Set["TensorVariable"], Set[Union[Variable, "TensorVariable"]]]
-        ] = None,
-        profile: Optional[bool] = None,
+        no_recycling: Optional[Sequence[Variable]] = None,
+        profile: Optional[Union[bool, "ProfileStats"]] = None,
     ) -> "PerformLinker":
-        """
+        """Associate a `FunctionGraph` with this `Linker`.
 
         Parameters
         ----------
-        fgraph : :py:class:`aesara.graph.fg.FunctionGraph` instance
-            A :py:class:`aesara.link.basic.PerformLinker` instance can have accepted
-            one :py:class:`aesara.graph.fg.FunctionGraph` instance at a time.
+        fgraph
+            A `PerformLinker` instance can have accepted one `FunctionGraph`
+            instance at a time.
         no_recycling
             WRITEME
 
-        Returns
-        -------
-        object
-            self (TODO: WHY? Who calls this function?)
-
         """
         if no_recycling is None:
             no_recycling = []
@@ -349,32 +327,10 @@ class PerformLinker(LocalLinker):
 
     def make_all(
         self,
-        input_storage: Optional[Any] = None,
-        output_storage: Optional[OutputStorageType] = None,
-        storage_map: Optional[StorageMapType] = None,
-    ) -> Union[
-        Tuple[Callable, List[Container], List[Container], List[Callable], List[Apply]],
-        Tuple[Callable, List[Container], List[Container], List[Any], List[Any]],
-    ]:
-        """
-        Returns Function to run all nodes, list of input containers, list of outputs
-
-        Parameters
-        ----------
-        input_storage : optional, list
-            list of storages corresponding to fgraph.inputs
-        output_storage
-            list of storages corresponding to fgraph.outputs
-        storage_map : iterable
-
-        Returns
-        -------
-        object
-            Function to run all nodes, list of input containers, list of output
-            containers, list of thunks (for all programs), list of nodes
-            (for all programs).
-
-        """
+        input_storage=None,
+        output_storage=None,
+        storage_map=None,
+    ):
         fgraph: Any = self.fgraph
         order = self.schedule(fgraph)
         no_recycling = self.no_recycling
@@ -461,7 +417,8 @@ class WrapLinker(Linker):
 
     Parameters
     ----------
-    linkers : list of L{LocalLinker} subclasses, whose make_all() method returns
+    linkers
+        List of L{LocalLinker} subclasses, whose make_all() method returns
         thunks in the same order.
         For each node in the graph, each linker will provide a
         thunk.  This class makes it possible to iterate over each linker's
@@ -484,10 +441,10 @@ class WrapLinker(Linker):
 
     def __init__(
         self,
-        linkers: Union[List["OpWiseCLinker"], List[PerformLinker], List["VMLinker"]],
+        linkers: Sequence[PerformLinker],
         wrapper: Callable,
     ) -> None:
-        self.fgraph = None
+        self.fgraph: Optional[FunctionGraph] = None
         self.linkers = linkers
         self.wrapper = wrapper
 
@@ -520,8 +477,8 @@ class WrapLinker(Linker):
     def accept(
         self,
         fgraph: FunctionGraph,
-        no_recycling: Optional[Union[Set["TensorVariable"], List]] = None,
-        profile: None = None,
+        no_recycling: Optional[Sequence["TensorVariable"]] = None,
+        profile: Optional[Union[bool, "ProfileStats"]] = None,
     ) -> "WrapLinker":
         """
 
@@ -554,7 +511,7 @@ class WrapLinker(Linker):
     ) -> None:
         pass
 
-    def make_thunk(self, **kwargs) -> ThunkType:
+    def make_thunk(self, **kwargs):
         no_recycling = self.no_recycling
 
         make_all = [self.linkers[0].make_all(**kwargs)]
@@ -605,7 +562,7 @@ class WrapLinker(Linker):
 
 
 def WrapLinkerMany(
-    linkers: Union[List["OpWiseCLinker"], List["VMLinker"]], wrappers: List[Callable]
+    linkers: List[PerformLinker], wrappers: List[Callable]
 ) -> WrapLinker:
     """
     Variant on WrapLinker that runs a series of wrapper functions instead of
