@@ -3,9 +3,9 @@
 import copy
 import dataclasses
 import logging
-import warnings
 from collections import OrderedDict, namedtuple
-from typing import TYPE_CHECKING, Callable, List, Optional, Set, Tuple, Union
+from typing import TYPE_CHECKING, Callable, List, Optional, Sequence, Set, Tuple, Union
+from typing import cast as type_cast
 
 import numpy as np
 
@@ -20,11 +20,10 @@ from aesara.graph.basic import (
     equal_computations,
     graph_inputs,
 )
-from aesara.graph.fg import FunctionGraph
 from aesara.graph.op import get_test_value
-from aesara.graph.opt import TopoOptimizer, local_optimizer
+from aesara.graph.type import HasDataType
 from aesara.graph.utils import TestValueError
-from aesara.tensor.basic import AllocEmpty, get_scalar_constant_value
+from aesara.tensor.basic import AllocEmpty, cast
 from aesara.tensor.subtensor import set_subtensor
 from aesara.tensor.var import TensorConstant
 
@@ -57,14 +56,17 @@ def safe_new(
         nw_name = None
 
     if isinstance(x, Constant):
-        if dtype and x.dtype != dtype:
-            casted_x = x.astype(dtype)
+        # TODO: Do something better about this
+        assert isinstance(x.type, HasDataType)
+
+        if dtype and x.type.dtype != dtype:
+            casted_x = cast(x, dtype)
             nwx = type(x)(casted_x.type, x.data, x.name)
             nwx.tag = copy.copy(x.tag)
             return nwx
         else:
-            return x.clone()
-    # Note, `as_tensor_variable` will convert the `Scalar` into a
+            return x
+    # Note, `as_tensor_variable` will convert the `ScalarType` into a
     # `TensorScalar` that will require a `ScalarFromTensor` `Op`, making the
     # push-out optimization fail
     elif isinstance(x, aes.ScalarVariable):
@@ -91,8 +93,12 @@ def safe_new(
             pass
 
     # Cast `x` if needed. If `x` has a test value, this will also cast it.
-    if dtype and x.dtype != dtype:
-        x = x.astype(dtype)
+    if dtype:
+        # TODO: Do something better about this
+        assert isinstance(x.type, HasDataType)
+
+        if x.type.dtype != dtype:
+            x = cast(x, dtype)
 
     nw_x = x.type()
     nw_x.name = nw_name
@@ -194,11 +200,6 @@ def traverse(out, x, x_copy, d, visited=None):
 
     There are two options :
         1) x and x_copy or on host, then you would replace x with x_copy
-        2) x is on gpu, x_copy on host, then you need to replace
-        host_from_gpu(x) with x_copy
-    This happens because initially shared variables are on GPU... which is
-    fine for the main computational graph but confuses things a bit for the
-    inner graph of scan.
 
     """
     # ``visited`` is a set of nodes that are already known and don't need to be
@@ -210,386 +211,20 @@ def traverse(out, x, x_copy, d, visited=None):
     if out in visited:
         return d
     visited.add(out)
-    from aesara.gpuarray import pygpu_activated
-    from aesara.gpuarray.basic_ops import GpuFromHost, host_from_gpu
-    from aesara.gpuarray.type import GpuArrayType
 
     if out == x:
-        assert isinstance(x.type, GpuArrayType)
-        d[out] = GpuFromHost(x.type.context_name)(x_copy)
-        return d
+        # assert isinstance(x.type, GpuArrayType)
+        # d[out] = GpuFromHost(x.type.context_name)(x_copy)
+        # return d
+        raise RuntimeError("Not supported")
     elif out.owner is None:
         return d
-    elif pygpu_activated and out.owner.op == host_from_gpu and out.owner.inputs == [x]:
-        d[out] = at.as_tensor_variable(x_copy)
-        return d
     else:
         for inp in out.owner.inputs:
             d = traverse(inp, x, x_copy, d, visited)
         return d
 
 
-def map_variables(replacer, graphs, additional_inputs=None):
-    """Construct new graphs based on 'graphs' with some variables replaced
-    according to 'replacer'.
-
-    :param replacer: function that takes a variable and returns its
-         replacement.
-    :param graphs: an iterable of graphs in which to replace variables
-    :param additional_inputs: an iterable of graph inputs not used in any
-         of 'graphs' but possibly used in the graphs returned by `replacer`
-    :return: the new graphs, in the same order as 'graphs'
-
-    Example:
-
-    .. code-block:: python
-
-        tag = "replaceme"
-
-        a = aesara.tensor.type.scalar("a")
-        b = aesara.tensor.type.scalar("b")
-        c = aesara.tensor.type.scalar("c")
-
-        ab = a + b
-        ab.tag.replacement = a * b
-
-        u = ab + c
-        v, = map_variables(lambda graph:
-            return getattr(graph.tag, "replacement", graph),
-            [u])
-
-        # v is now equal to a * b + c
-    """
-    if additional_inputs is None:
-        additional_inputs = []
-
-    # wrap replacer to avoid replacing things we just put there.
-    graphs_seen = set()
-
-    def wrapped_replacer(graph):
-        if graph in graphs_seen:
-            return graph
-        else:
-            new_graph = replacer(graph)
-            graphs_seen.add(new_graph)
-            return new_graph
-
-    graphs = list(graphs)
-    inputs_ = list(set(list(graph_inputs(graphs)) + list(additional_inputs)))
-
-    # perform any desired replacement of input variables.  these
-    # aren't replaced by the local optimizer approach because they are
-    # not outputs of any Apply node.
-    new_inputs = [wrapped_replacer(i) for i in inputs_]
-    replacements = [
-        (input_, new_input)
-        for input_, new_input in zip(inputs_, new_inputs)
-        if new_input is not input_
-    ]
-    graphs = clone_replace(graphs, share_inputs=True, replace=replacements)
-    inputs_ = list(set(list(graph_inputs(graphs)) + list(additional_inputs)))
-
-    fg = FunctionGraph(inputs_, graphs, clone=False)
-
-    nodes_seen = set()
-
-    @local_optimizer(None)
-    def local_transform(fgraph, node):
-        if node in nodes_seen:
-            return False
-
-        # importing Scan into module scope would be circular
-        from aesara.compile.builders import OpFromGraph
-        from aesara.scan.op import Scan
-
-        if isinstance(node.op, (Scan, OpFromGraph)):
-            # recurse on the inner graph
-            (
-                new_inner_inputs,
-                new_outer_inputs,
-                new_inner_outputs,
-            ) = _map_variables_inner(
-                wrapped_replacer,
-                inner_inputs=node.op.inputs,
-                outer_inputs=node.inputs,
-                inner_outputs=node.op.outputs,
-                containing_op=node.op,
-            )
-            # reinstantiate the op
-            if isinstance(node.op, Scan):
-                new_op = Scan(
-                    new_inner_inputs,
-                    new_inner_outputs,
-                    node.op.info,
-                    node.op.mode,
-                    # FIXME: infer this someday?
-                    typeConstructor=None,
-                )
-            elif isinstance(node.op, OpFromGraph):
-                new_op = OpFromGraph(
-                    new_inner_inputs, new_inner_outputs, **node.op.kwargs
-                )
-            # make a new node to replace the old one
-            new_node = new_op.make_node(*new_outer_inputs)
-            nodes_seen.add(new_node)
-            return new_node.outputs
-        else:
-            nodes_seen.add(node)
-            replacements = [wrapped_replacer(o) for o in node.outputs]
-
-            # Add inputs to replacement graphs as inputs to this `fgraph`
-            for i in graph_inputs(replacements):
-                fgraph.add_input(i)
-
-            return replacements
-
-    topo_transform = TopoOptimizer(local_transform, "out_to_in")
-    topo_transform.optimize(fg)
-
-    new_graphs = fg.outputs
-    fg.disown()
-    return new_graphs
-
-
-def _map_variables_inner(
-    replacer, inner_inputs, outer_inputs, inner_outputs, containing_op
-):
-    # the replacements returned by the replacer may involve variables
-    # that are already owned by the outer fgraph (`fg` in the caller)
-    # and so cannot be added to the inner fgraph (`fg` in the
-    # recursive call).  wrap the replacer to catch these before they
-    # are added.
-
-    # additionally, some of these may be fgraph inputs or shared
-    # variables, which we cannot directly use inside the inner graph.
-    # we need to create inner inputs to access them through.
-
-    outer_to_inner = dict(zip(outer_inputs, inner_inputs))
-    extra_inner_inputs = []
-    extra_outer_inputs = []
-
-    from itertools import chain
-
-    from aesara.scan import utils
-
-    def inner_replacer(graph):
-        new_graph = replacer(graph)
-
-        other_inputs = []
-        constants = []
-        for input_ in graph_inputs([new_graph]):
-            if isinstance(input_, Variable):
-                if isinstance(input_, Constant):
-                    constants.append(input_)
-                else:
-                    other_inputs.append(input_)
-
-        # foreign inputs are fgraph inputs and shared variables that we need
-        # to access through inner inputs
-        foreign_inputs = list(set(other_inputs) - set(outer_to_inner.values()))
-
-        # skip further processing if there is nothing to do
-        if not constants and not foreign_inputs:
-            return new_graph
-
-        replacements = []
-
-        # constants just need to be replaced by copies that the inner
-        # `fg` can take ownership of
-        for input_ in constants:
-            new_input = input_.clone()
-            new_input.name = f"{new_input.name}_copied"
-            replacements.append((input_, new_input))
-
-        for outer_input in foreign_inputs:
-            if getattr(outer_input, "update", False):
-                # when aesara.scan() constructs a scan node, it detects
-                # shared variables with updates and returns these updates
-                # to the user.  we need to do the same thing for every new
-                # use of such a variable that is introduced.  it's hard to
-                # do that at this point.
-                # shared variables with updates inside the inner graph of
-                # OpFromGraph are not supported at all, so we don't support
-                # introducing those either.
-                raise NotImplementedError(
-                    f"Replacement introduces shared variable {outer_input} "
-                    "which has an update associated with it into "
-                    f"the inner graph of {containing_op}. This is not currently "
-                    "supported."
-                )
-            # if this foreign input is not already available
-            # as an inner input, connect it through a new
-            # inner input
-            if outer_input not in outer_to_inner.keys():
-                inner_input = utils.safe_new(outer_input, tag="_copy")
-                outer_to_inner[outer_input] = inner_input
-                extra_inner_inputs.append(inner_input)
-                extra_outer_inputs.append(outer_input)
-
-        replacements.extend(outer_to_inner.items())
-
-        (new_graph,) = clone_replace(
-            [new_graph], share_inputs=True, replace=replacements
-        )
-        return new_graph
-
-    new_inner_outputs = map_variables(inner_replacer, inner_outputs)
-    new_inner_inputs = list(chain(inner_inputs, extra_inner_inputs))
-    new_outer_inputs = list(chain(outer_inputs, extra_outer_inputs))
-
-    return new_inner_inputs, new_outer_inputs, new_inner_outputs
-
-
-def get_updates_and_outputs(ls):
-    """
-    This function tries to recognize the updates OrderedDict, the
-    list of outputs and the stopping condition returned by the
-    lambda expression and arrange them in a predefined order.
-
-    WRITEME: what is the type of ls? how is it formatted?  if it's not in the
-    predefined order already, how does this function know how to put it in that
-    order?
-
-    """
-
-    def is_outputs(elem):
-        if isinstance(elem, (list, tuple)) and all(
-            [isinstance(x, Variable) for x in elem]
-        ):
-            return True
-        if isinstance(elem, Variable):
-            return True
-        return False
-
-    def is_updates(elem):
-        if isinstance(elem, dict):
-            # Make sure the updates will be applied in a deterministic order
-            if not isinstance(elem, OrderedDict) and len(elem) > 1:
-                warnings.warn(
-                    "Expected OrderedDict or OrderedUpdates, got "
-                    + str(type(elem))
-                    + ". This can make your script non-"
-                    "deterministic."
-                )
-            return True
-        # Dictionaries can be given as lists of tuples
-        if isinstance(elem, (list, tuple)) and all(
-            [isinstance(x, (list, tuple)) and len(x) == 2 for x in elem]
-        ):
-            return True
-        return False
-
-    def is_condition(elem):
-        return isinstance(elem, until)
-
-    def _list(x):
-        if isinstance(x, (list, tuple)):
-            return list(x)
-        else:
-            return [x]
-
-    def _filter(x):
-        """
-        Ensure `x` is made only of allowed data types.
-
-        Return True iff `x` is made only of lists, tuples, dictionaries, Aesara
-        variables or `aesara.scan.utils.until` objects.
-
-        """
-        # Is `x` a container we can iterate on?
-        iter_on = None
-        if isinstance(x, list) or isinstance(x, tuple):
-            iter_on = x
-        elif isinstance(x, dict):
-            iter_on = x.items()
-        if iter_on is not None:
-            return all(_filter(y) for y in iter_on)
-        else:
-            return isinstance(x, Variable) or isinstance(x, until)
-
-    if not _filter(ls):
-        raise ValueError(
-            "The return value of your scan lambda expression may only be "
-            "made of lists, tuples, or dictionaries containing Aesara "
-            "variables (or `aesara.scan.utils.until` objects for "
-            "conditions). In particular if you need to use constant "
-            "values, you can use `tensor.constant` to turn them into "
-            "Aesara variables."
-        )
-
-    if is_outputs(ls):
-        return None, _list(ls), OrderedDict()
-    if is_updates(ls):
-        return None, [], OrderedDict(ls)
-    error_msg = (
-        f"Scan cannot parse the return value of your lambda expression, which is: {ls}"
-    )
-    if not isinstance(ls, (list, tuple)):
-        raise ValueError(error_msg)
-    ls = list(ls)
-    deprecation_msg = (
-        "The return value of the lambda function"
-        " has been restricted. you have to always return first the"
-        " outputs (if any), afterwards the updates (if any) and"
-        " at the end the conclusion"
-    )
-    if len(ls) == 2:
-        if is_outputs(ls[0]):
-            if is_updates(ls[1]):
-                return (None, _list(ls[0]), OrderedDict(ls[1]))
-            elif is_condition(ls[1]):
-                return (ls[1].condition, _list(ls[0]), OrderedDict())
-            else:
-                raise ValueError(error_msg)
-        elif is_updates(ls[0]):
-            if is_outputs(ls[1]):
-                raise ValueError(deprecation_msg)
-            elif is_condition(ls[1]):
-                return (ls[1].condition, [], OrderedDict(ls[0]))
-            else:
-                raise ValueError(error_msg)
-        else:
-            raise ValueError(error_msg)
-    elif len(ls) == 3:
-        if is_outputs(ls[0]):
-            if is_updates(ls[1]):
-                if is_condition(ls[2]):
-                    return (ls[2].condition, _list(ls[0]), OrderedDict(ls[1]))
-                else:
-                    raise ValueError(error_msg)
-            else:
-                raise ValueError(error_msg)
-        else:
-            raise ValueError(error_msg)
-    else:
-        raise ValueError(error_msg)
-
-
-def isNaN_or_Inf_or_None(x):
-    isNone = x is None
-    try:
-        isNaN = np.isnan(x)
-        isInf = np.isinf(x)
-        isStr = isinstance(x, str)
-    except Exception:
-        isNaN = False
-        isInf = False
-        isStr = False
-    if not isNaN and not isInf:
-        try:
-            val = get_scalar_constant_value(x)
-            isInf = np.isinf(val)
-            isNaN = np.isnan(val)
-        except Exception:
-            isNaN = False
-            isInf = False
-    if isinstance(x, Constant) and isinstance(x.data, str):
-        isStr = True
-    else:
-        isStr = False
-    return isNone or isNaN or isInf or isStr
-
-
 def expand_empty(tensor_var, size):
     """
     Transforms the shape of a tensor from (d1, d2 ... ) to ( d1+size, d2, ..)
@@ -910,14 +545,8 @@ def reconstruct_graph(inputs, outputs, tag=None):
     if tag is None:
         tag = ""
     nw_inputs = [safe_new(x, tag) for x in inputs]
-    givens = OrderedDict()
-    for nw_x, x in zip(nw_inputs, inputs):
-        givens[x] = nw_x
-    allinputs = list(graph_inputs(outputs))
-    for inp in allinputs:
-        if isinstance(inp, Constant):
-            givens[inp] = inp.clone()
 
+    givens = {x: nw_x for nw_x, x in zip(nw_inputs, inputs)}
     nw_outputs = clone_replace(outputs, replace=givens)
     return (nw_inputs, nw_outputs)
 
@@ -946,13 +575,13 @@ class ScanArgs:
 
     def __init__(
         self,
-        outer_inputs,
-        outer_outputs,
-        _inner_inputs,
-        _inner_outputs,
-        info,
-        as_while,
-        clone=True,
+        outer_inputs: Sequence[Variable],
+        outer_outputs: Sequence[Variable],
+        _inner_inputs: Sequence[Variable],
+        _inner_outputs: Sequence[Variable],
+        info: "ScanInfo",
+        as_while: bool,
+        clone: Optional[bool] = True,
     ):
         self.n_steps = outer_inputs[0]
         self.as_while = as_while
@@ -974,16 +603,16 @@ class ScanArgs:
         q = 0
 
         n_seqs = info.n_seqs
-        self.outer_in_seqs = outer_inputs[p : p + n_seqs]
-        self.inner_in_seqs = inner_inputs[q : q + n_seqs]
+        self.outer_in_seqs = list(outer_inputs[p : p + n_seqs])
+        self.inner_in_seqs = list(inner_inputs[q : q + n_seqs])
         p += n_seqs
         q += n_seqs
 
         n_mit_mot = info.n_mit_mot
         n_mit_sot = info.n_mit_sot
 
-        self.mit_mot_in_slices = info.tap_array[:n_mit_mot]
-        self.mit_sot_in_slices = info.tap_array[n_mit_mot : n_mit_mot + n_mit_sot]
+        self.mit_mot_in_slices = list(info.tap_array[:n_mit_mot])
+        self.mit_sot_in_slices = list(info.tap_array[n_mit_mot : n_mit_mot + n_mit_sot])
 
         n_mit_mot_ins = sum(len(s) for s in self.mit_mot_in_slices)
         n_mit_sot_ins = sum(len(s) for s in self.mit_sot_in_slices)
@@ -1004,29 +633,29 @@ class ScanArgs:
             qq += len(sl)
         q += n_mit_sot_ins
 
-        self.outer_in_mit_mot = outer_inputs[p : p + n_mit_mot]
+        self.outer_in_mit_mot = list(outer_inputs[p : p + n_mit_mot])
         p += n_mit_mot
-        self.outer_in_mit_sot = outer_inputs[p : p + n_mit_sot]
+        self.outer_in_mit_sot = list(outer_inputs[p : p + n_mit_sot])
         p += n_mit_sot
 
         n_sit_sot = info.n_sit_sot
-        self.outer_in_sit_sot = outer_inputs[p : p + n_sit_sot]
-        self.inner_in_sit_sot = inner_inputs[q : q + n_sit_sot]
+        self.outer_in_sit_sot = list(outer_inputs[p : p + n_sit_sot])
+        self.inner_in_sit_sot = list(inner_inputs[q : q + n_sit_sot])
         p += n_sit_sot
         q += n_sit_sot
 
         n_shared_outs = info.n_shared_outs
-        self.outer_in_shared = outer_inputs[p : p + n_shared_outs]
-        self.inner_in_shared = inner_inputs[q : q + n_shared_outs]
+        self.outer_in_shared = list(outer_inputs[p : p + n_shared_outs])
+        self.inner_in_shared = list(inner_inputs[q : q + n_shared_outs])
         p += n_shared_outs
         q += n_shared_outs
 
         n_nit_sot = info.n_nit_sot
-        self.outer_in_nit_sot = outer_inputs[p : p + n_nit_sot]
+        self.outer_in_nit_sot = list(outer_inputs[p : p + n_nit_sot])
         p += n_nit_sot
 
-        self.outer_in_non_seqs = outer_inputs[p:]
-        self.inner_in_non_seqs = inner_inputs[q:]
+        self.outer_in_non_seqs = list(outer_inputs[p:])
+        self.inner_in_non_seqs = list(inner_inputs[q:])
 
         # now for the outputs
         p = 0
@@ -1034,9 +663,9 @@ class ScanArgs:
 
         self.mit_mot_out_slices = info.mit_mot_out_slices
         n_mit_mot_outs = info.n_mit_mot_outs
-        self.outer_out_mit_mot = outer_outputs[p : p + n_mit_mot]
-        iomm = inner_outputs[q : q + n_mit_mot_outs]
-        self.inner_out_mit_mot = ()
+        self.outer_out_mit_mot = list(outer_outputs[p : p + n_mit_mot])
+        iomm = list(inner_outputs[q : q + n_mit_mot_outs])
+        self.inner_out_mit_mot: Tuple[List[Variable], ...] = ()
         qq = 0
         for sl in self.mit_mot_out_slices:
             self.inner_out_mit_mot += (iomm[qq : qq + len(sl)],)
@@ -1044,23 +673,23 @@ class ScanArgs:
         p += n_mit_mot
         q += n_mit_mot_outs
 
-        self.outer_out_mit_sot = outer_outputs[p : p + n_mit_sot]
-        self.inner_out_mit_sot = inner_outputs[q : q + n_mit_sot]
+        self.outer_out_mit_sot = list(outer_outputs[p : p + n_mit_sot])
+        self.inner_out_mit_sot = list(inner_outputs[q : q + n_mit_sot])
         p += n_mit_sot
         q += n_mit_sot
 
-        self.outer_out_sit_sot = outer_outputs[p : p + n_sit_sot]
-        self.inner_out_sit_sot = inner_outputs[q : q + n_sit_sot]
+        self.outer_out_sit_sot = list(outer_outputs[p : p + n_sit_sot])
+        self.inner_out_sit_sot = list(inner_outputs[q : q + n_sit_sot])
         p += n_sit_sot
         q += n_sit_sot
 
-        self.outer_out_nit_sot = outer_outputs[p : p + n_nit_sot]
-        self.inner_out_nit_sot = inner_outputs[q : q + n_nit_sot]
+        self.outer_out_nit_sot = list(outer_outputs[p : p + n_nit_sot])
+        self.inner_out_nit_sot = list(inner_outputs[q : q + n_nit_sot])
         p += n_nit_sot
         q += n_nit_sot
 
-        self.outer_out_shared = outer_outputs[p : p + n_shared_outs]
-        self.inner_out_shared = inner_outputs[q : q + n_shared_outs]
+        self.outer_out_shared = list(outer_outputs[p : p + n_shared_outs])
+        self.inner_out_shared = list(inner_outputs[q : q + n_shared_outs])
         p += n_shared_outs
         q += n_shared_outs
 
@@ -1083,25 +712,6 @@ class ScanArgs:
             clone=clone,
         )
 
-    @classmethod
-    def create_empty(cls) -> "ScanArgs":
-        from aesara.scan.op import ScanInfo
-
-        info = ScanInfo(
-            n_seqs=0,
-            n_mit_mot=0,
-            n_mit_sot=0,
-            tap_array=(),
-            n_sit_sot=0,
-            n_nit_sot=0,
-            n_shared_outs=0,
-            n_mit_mot_outs=0,
-            mit_mot_out_slices=(),
-        )
-        res = cls([1], [], [], [], info, False)
-        res.n_steps = None
-        return res
-
     @property
     def n_nit_sot(self):
         # This is just a hack that allows us to use `Scan.get_oinp_iinp_iout_oout_mappings`
@@ -1226,16 +836,22 @@ class ScanArgs:
         -------
         The alternate variable.
         """
+        _var_info: FieldInfo
         if not isinstance(var_info, FieldInfo):
-            var_info = self.find_among_fields(var_info)
+            find_var_info = self.find_among_fields(var_info)
+            if find_var_info is None:
+                raise ValueError(f"Couldn't find {var_info} among fields")
+            _var_info = find_var_info
+        else:
+            _var_info = var_info
 
-        alt_type = var_info.name[(var_info.name.index("_", 6) + 1) :]
-        alt_var = getattr(self, f"{alt_prefix}_{alt_type}")[var_info.index]
-        return alt_var
+        alt_type = _var_info.name[(_var_info.name.index("_", 6) + 1) :]
+        alt_var = getattr(self, f"{alt_prefix}_{alt_type}")[_var_info.index]
+        return type_cast(Variable, alt_var)
 
     def find_among_fields(
         self, i: Variable, field_filter: Callable[[str], bool] = default_filter
-    ) -> Optional[Tuple[str, int, int, int]]:
+    ) -> Optional[FieldInfo]:
         """Find the type and indices of the field containing a given element.
 
         NOTE: This only returns the *first* field containing the given element.
@@ -1302,7 +918,7 @@ class ScanArgs:
         return field_info
 
     def get_dependent_nodes(
-        self, i: Variable, seen: Optional[Set[int]] = None
+        self, i: Variable, seen: Optional[Set[Variable]] = None
     ) -> Set[Variable]:
         if seen is None:
             seen = {i}
@@ -1377,14 +993,14 @@ class ScanArgs:
 
     def remove_from_fields(
         self, i: Variable, rm_dependents: bool = True
-    ) -> List[FieldInfo]:
+    ) -> List[Tuple[Variable, Optional[FieldInfo]]]:
 
         if rm_dependents:
             vars_to_remove = self.get_dependent_nodes(i) | {i}
         else:
             vars_to_remove = {i}
 
-        rm_info = []
+        rm_info: List[Tuple[Variable, Optional[FieldInfo]]] = []
         for v in vars_to_remove:
             dependent_rm_info = self._remove_from_fields(v)
             rm_info.append((v, dependent_rm_info))
