@@ -320,15 +320,16 @@ class Elemwise(OpenMPOp):
 
     Notes
     -----
-    | Elemwise(add) represents + on tensors (x + y)
-    | Elemwise(add, {0 : 0}) represents the += operation (x += y)
-    | Elemwise(add, {0 : 1}) represents += on the second argument (y += x)
-    | Elemwise(mul)(rand(10, 5), rand(1, 5)) the second input is completed \
-along the first dimension to match the first input
-    | Elemwise(true_div)(rand(10, 5), rand(10, 1)) same but along the \
-second dimension
-    | Elemwise(int_div)(rand(1, 5), rand(10, 1)) the output has size (10, 5)
-    | Elemwise(log)(rand(3, 4, 5))
+    -``Elemwise(add)``: represents ``+`` on tensors ``x + y``
+    -``Elemwise(add, {0 : 0})``: represents the ``+=`` operation ``x += y``
+    -``Elemwise(add, {0 : 1})``: represents ``+=`` on the second argument ``y += x``
+    -``Elemwise(mul)(np.random.random((10, 5)), np.random.random((1, 5)))``:
+    the second input is completed along the first dimension to match the first input
+    -``Elemwise(true_div)(np.random.random(10, 5), np.random.random(10, 1))``: same but along the
+    second dimension
+    -``Elemwise(int_div)(np.random.random((1, 5)), np.random.random((10, 1)))``:
+    the output has size ``(10, 5)``.
+    -``Elemwise(log)(np.random.random((3, 4, 5)))``
 
     """
 
@@ -393,7 +394,7 @@ second dimension
             *[get_scalar_type(dtype=i.type.dtype).make_variable() for i in inputs]
         )
 
-        target_length = max([input.type.ndim for input in inputs])
+        target_length = max(input.type.ndim for input in inputs)
 
         args = []
         for input in inputs:
