@@ -449,8 +449,6 @@ def cumprod(x, axis=None):
     return CumOp(axis=axis, mode="mul")(x)
 
 
-# CumsumOp and CumprodOp are for compatibility with old version,
-# just in case unpickling an Aesara function with old Ops.
 class CumsumOp(Op):
     __props__ = ("axis",)
 
@@ -588,19 +586,21 @@ def squeeze(x, axis=None):
     `x` without `axis` dimensions.
 
     """
+    _x = at.as_tensor_variable(x)
+
     if axis is None:
         # By default exclude all broadcastable (length=1) axes
-        axis = (i for i in range(x.ndim) if x.broadcastable[i])
+        axis = (i for i in range(_x.ndim) if _x.broadcastable[i])
     elif not isinstance(axis, Collection):
         axis = (axis,)
 
     # scalar inputs are treated as 1D regarding axis in this `Op`
     try:
-        axis = np.core.numeric.normalize_axis_tuple(axis, ndim=max(1, x.ndim))
+        axis = np.core.numeric.normalize_axis_tuple(axis, ndim=max(1, _x.ndim))
     except np.AxisError:
-        raise np.AxisError(axis, ndim=x.ndim)
+        raise np.AxisError(axis, ndim=_x.ndim)
 
-    return x.dimshuffle([i for i in range(x.ndim) if i not in axis])
+    return _x.dimshuffle([i for i in range(_x.ndim) if i not in axis])
 
 
 def compress(condition, x, axis=None):
@@ -628,8 +628,9 @@ def compress(condition, x, axis=None):
     `x` with selected slices.
 
     """
+    _x = at.as_tensor_variable(x)
     indices = at.flatnonzero(condition)
-    return x.take(indices, axis=axis)
+    return _x.take(indices, axis=axis)
 
 
 class Repeat(Op):
@@ -846,7 +847,6 @@ class Bartlett(Op):
 bartlett_ = Bartlett()
 
 
-# I create a function only to have the doc show well.
 def bartlett(M):
     """
     An instance of this class returns the Bartlett spectral window in the
@@ -942,7 +942,6 @@ class FillDiagonal(Op):
 fill_diagonal_ = FillDiagonal()
 
 
-# I create a function only to have the doc show well.
 def fill_diagonal(a, val):
     """
     Returns a copy of an array with all elements of the main diagonal set to a
@@ -1498,12 +1497,14 @@ def broadcast_shape_iter(
     else:
         max_dims = max(a.ndim for a in arrays)
 
+        _arrays = tuple(at.as_tensor_variable(a) for a in arrays)
+
         array_shapes = [
             (one_at,) * (max_dims - a.ndim)
             + tuple(
-                one_at if bcast else sh for sh, bcast in zip(a.shape, a.broadcastable)
+                one_at if t_sh == 1 else sh for sh, t_sh in zip(a.shape, a.type.shape)
             )
-            for a in arrays
+            for a in _arrays
         ]
 
     result_dims = []
@@ -1666,3 +1667,27 @@ def broadcast_arrays(*args: TensorVariable) -> Tuple[TensorVariable, ...]:
 
     """
     return tuple(broadcast_to(a, broadcast_shape(*args)) for a in args)
+
+
+__all__ = [
+    "searchsorted",
+    "cumsum",
+    "cumprod",
+    "diff",
+    "bincount",
+    "squeeze",
+    "compress",
+    "repeat",
+    "bartlett",
+    "fill_diagonal",
+    "fill_diagonal_offset",
+    "unique",
+    "unravel_index",
+    "ravel_multi_index",
+    "broadcast_shape",
+    "broadcast_to",
+    "geomspace",
+    "logspace",
+    "linspace",
+    "broadcast_arrays",
+]
