@@ -1,20 +1,24 @@
 import warnings
-from typing import Dict
+from numbers import Number
+from textwrap import dedent
+from typing import Dict, List, Tuple, Union
 
 import numpy as np
 
 import aesara
 from aesara.gradient import DisconnectedType
 from aesara.graph.basic import Apply, Variable
-from aesara.graph.op import COp
-from aesara.graph.params_type import ParamsType
+from aesara.link.c.op import COp
+from aesara.link.c.params_type import ParamsType
 from aesara.misc.safe_asarray import _asarray
 from aesara.scalar import int32
 from aesara.tensor import _get_vector_length
-from aesara.tensor import basic as aet
+from aesara.tensor import basic as at
+from aesara.tensor import get_vector_length
 from aesara.tensor.exceptions import NotScalarConstantError
-from aesara.tensor.type import TensorType, int_dtypes, tensor
-from aesara.tensor.var import TensorConstant
+from aesara.tensor.type import DenseTensorType, TensorType, int_dtypes, tensor
+from aesara.tensor.type_other import NoneConst
+from aesara.tensor.var import TensorConstant, TensorVariable
 
 
 def register_shape_c_code(type, code, version=()):
@@ -57,11 +61,15 @@ class Shape(COp):
     __props__ = ()
 
     def make_node(self, x):
-        # Must work for all type that have a shape attribute.
-        # This will fail at execution time.
         if not isinstance(x, Variable):
-            x = aet.as_tensor_variable(x)
-        return Apply(self, [x], [aesara.tensor.type.lvector()])
+            x = at.as_tensor_variable(x)
+
+        if isinstance(x.type, TensorType):
+            out_var = TensorType("int64", (x.type.ndim,))()
+        else:
+            out_var = aesara.tensor.type.lvector()
+
+        return Apply(self, [x], [out_var])
 
     def perform(self, node, inp, out_):
         (x,) = inp
@@ -126,8 +134,22 @@ class Shape(COp):
         return tuple(version)
 
 
-shape = Shape()
-_shape = shape  # was used in the past, now use shape directly.
+_shape = Shape()
+
+
+def shape(x: Union[np.ndarray, Number, Variable]) -> Variable:
+    """Return the shape of `x`."""
+    if not isinstance(x, Variable):
+        x = at.as_tensor_variable(x)
+
+    x_type = x.type
+
+    if isinstance(x_type, TensorType) and all(s is not None for s in x_type.shape):
+        res = at.as_tensor_variable(x_type.shape, ndim=1, dtype=np.int64)
+    else:
+        res = _shape(x)
+
+    return res
 
 
 @_get_vector_length.register(Shape)
@@ -135,7 +157,7 @@ def _get_vector_length_Shape(op, var):
     return var.owner.inputs[0].type.ndim
 
 
-def shape_tuple(x):
+def shape_tuple(x: TensorVariable) -> Tuple[Variable, ...]:
     """Get a tuple of symbolic shape values.
 
     This will return a `ScalarConstant` with the value ``1`` wherever
@@ -144,7 +166,9 @@ def shape_tuple(x):
     one_at = aesara.scalar.ScalarConstant(aesara.scalar.int64, 1)
     return tuple(
         one_at if getattr(sh, "value", sh) == 1 or bcast else sh
-        for sh, bcast in zip(shape(x), getattr(x, "broadcastable", (False,) * x.ndim))
+        for sh, bcast in zip(
+            shape(x), getattr(x, "broadcastable", (False,) * x.type.ndim)
+        )
     )
 
 
@@ -194,9 +218,11 @@ class Shape_i(COp):
         return "%s{%i}" % (self.__class__.__name__, self.i)
 
     def make_node(self, x):
-        if not isinstance(x, Variable):
-            raise TypeError(f"{x} must be Variable with ndim attribute")
-        if x.ndim <= self.i:
+        if not isinstance(x, Variable) or not hasattr(x.type, "ndim"):
+            raise TypeError(
+                f"{x} must be `Variable` with a `Type` having an ndim attribute"
+            )
+        if x.type.ndim <= self.i:
             raise TypeError(f"{x} has too few dimensions for Shape_i")
         return Apply(self, [x], [aesara.tensor.type.lscalar()])
 
@@ -338,28 +364,6 @@ def register_shape_i_c_code(typ, code, check_input, version=()):
     Shape_i.c_code_and_version[typ] = (code, check_input, version)
 
 
-def register_specify_shape_c_code(typ, code, version=(), c_support_code_apply=None):
-    """
-    Tell SpecifyShape how to generate C code for an Aesara Type.
-
-    Parameters
-    ----------
-    typ : Aesara type
-        It must be the Aesara class itself and not an instance of the class.
-    code : C code
-        Checks the shape and returns a view for the Aesara type 'typ'.
-        Use %(iname)s and %(oname)s for the input and output C variable names
-        respectively. %(shape)s is the vector of shape of %(iname)s.
-        Check that its length is good.
-    version
-        A number indicating the version of the code, for cache.
-    c_support_code_apply
-        Extra code.
-
-    """
-    SpecifyShape.c_code_and_version[typ] = (code, version, c_support_code_apply)
-
-
 class SpecifyShape(COp):
     """
     L{Op} that puts into the graph the user-provided shape.
@@ -372,136 +376,193 @@ class SpecifyShape(COp):
     Notes
     -----
     Maybe in the future we will never do the assert!
-
-    We currently don't support specifying partial shape information.
-
-    TODO : test this op with sparse. Do C code for them too.
-
     """
 
     view_map = {0: [0]}
-    # Mapping from Type to C code (and version) to use.
-    # In the C code, the name of the input variable is %(iname)s,
-    # the output variable is %(oname)s.
-    c_code_and_version: Dict = {}
     __props__ = ()
     _f16_ok = True
 
-    def make_node(self, x, shape):
-        if not isinstance(x, Variable):
-            x = aet.as_tensor_variable(x)
-        if shape == () or shape == []:
-            tshape = aet.constant([], dtype="int64")
-        else:
-            tshape = aet.as_tensor_variable(shape, ndim=1)
-            if tshape.dtype not in aesara.tensor.type.integer_dtypes:
-                raise AssertionError(
-                    f"The `shape` must be an integer type. Got {tshape.dtype} instead."
-                )
-        if isinstance(tshape, TensorConstant) and tshape.data.size != x.ndim:
-            ndim = len(tshape.data)
-            raise AssertionError(
-                f"Input `x` is {x.ndim}-dimensional and will never match a {ndim}-dimensional shape."
+    def make_node(self, x, *shape):
+        from aesara.tensor.basic import get_scalar_constant_value
+
+        x = at.as_tensor_variable(x)
+
+        shape = tuple(
+            NoneConst
+            if (s is None or NoneConst.equals(s))
+            else at.as_tensor_variable(s, ndim=0)
+            for s in shape
+        )
+
+        if any(
+            s.dtype not in aesara.tensor.type.integer_dtypes
+            for s in shape
+            if hasattr(s, "dtype")
+        ):
+            raise TypeError("Shape values must be integer types")
+
+        if len(shape) != x.type.ndim:
+            raise ValueError(
+                f"Input `x` is {x.type.ndim}-dimensional and will never match a shape of length {len(shape)}."
             )
-        return Apply(self, [x, tshape], [x.type()])
+
+        type_shape = [None] * x.ndim
+        for i, (xts, s) in enumerate(zip(x.type.shape, shape)):
+            if xts is not None:
+                type_shape[i] = xts
+            else:
+                try:
+                    type_s = get_scalar_constant_value(s)
+                    if type_s is not None:
+                        type_shape[i] = int(type_s)
+                except NotScalarConstantError:
+                    pass
+
+        out_var = x.type.clone(shape=type_shape)()
+
+        return Apply(self, [x, *shape], [out_var])
 
     def perform(self, node, inp, out_):
-        x, shape = inp
+        x, *shape = inp
         (out,) = out_
         ndim = len(shape)
         if x.ndim != ndim:
             raise AssertionError(
                 f"SpecifyShape: Got {x.ndim} dimensions (shape {x.shape}), expected {ndim} dimensions with shape {tuple(shape)}."
             )
-        if not np.all(x.shape == shape):
+        if not all(xs == s for xs, s in zip(x.shape, shape) if s is not None):
             raise AssertionError(
-                f"SpecifyShape: Got shape {x.shape}, expected {tuple(shape)}."
+                f"SpecifyShape: Got shape {x.shape}, expected {tuple(int(s) if s is not None else None for s in shape)}."
             )
         out[0] = x
 
     def infer_shape(self, fgraph, node, shapes):
-        xshape, sshape = shapes
+        xshape, *_ = shapes
+        shape = node.inputs[1:]
         new_shape = []
-        for dim in range(node.inputs[0].ndim):
+        for dim in range(node.inputs[0].type.ndim):
+            s = shape[dim]
             try:
-                s = aet.get_scalar_constant_value(node.inputs[1][dim])
-                s = aet.as_tensor_variable(s)
-                new_shape.append(s)
+                s = at.get_scalar_constant_value(s)
+                # We assume that `None` shapes are always retrieved by
+                # `get_scalar_constant_value`, and only in that case do we default to
+                # the shape of the input variable
+                if s is None:
+                    s = xshape[dim]
             except NotScalarConstantError:
-                new_shape.append(node.inputs[1][dim])
+                pass
+            new_shape.append(at.as_tensor_variable(s))
 
         assert len(new_shape) == len(xshape)
         return [new_shape]
 
     def connection_pattern(self, node):
-        return [[True], [False]]
+        return [[True], *[[False]] * len(node.inputs[1:])]
 
     def grad(self, inp, grads):
-        x, s = inp
+        x, *shape = inp
         (gz,) = grads
         # Should I set an SpecifyShape on gz? I think so
         # But I don't do it now as we need to make an optimization
         # to remove that op from the graph to don't block other optimization
         # Should I do an optimizer that will remove the SpecifyShape?
         # I think Yes
-        return [gz, aesara.gradient.DisconnectedType()()]
-        return [specify_shape(gz, s), aesara.gradient.DisconnectedType()()]
+        # return [specify_shape(gz, s)] + [aesara.gradient.DisconnectedType()() for _ in range(len(shape))]
+        return [gz] + [aesara.gradient.DisconnectedType()() for _ in range(len(shape))]
 
     def R_op(self, inputs, eval_points):
         if eval_points[0] is None:
-            # It means that the this op sits on top of a non-differentiable
-            # path
+            # It means that this op sits on top of a non-differentiable path
             return [None]
         return self.make_node(eval_points[0], *inputs[1:]).outputs
 
-    def c_support_code_apply(self, node, name):
-        itype = node.inputs[0].type.__class__
-        if itype in self.c_code_and_version:
-            _, _, support_code = self.c_code_and_version[itype]
-            if support_code:
-                return support_code
-        return super().c_support_code_apply(node, name)
+    def c_code(self, node, name, i_names, o_names, sub):
+        if not isinstance(node.inputs[0].type, DenseTensorType):
+            raise NotImplementedError(
+                f"Specify_shape c_code not implemented for input type {node.inputs[0].type}"
+            )
 
-    def c_code(self, node, name, inames, onames, sub):
-        iname, shape = inames
-        (oname,) = onames
+        x_name, *shape_names = i_names
+        (o_name,) = o_names
         fail = sub["fail"]
 
-        itype = node.inputs[0].type.__class__
-        if itype in self.c_code_and_version:
-            code, version, _ = self.c_code_and_version[itype]
-            return code % locals()
+        code = dedent(
+            f"""
+            if (PyArray_NDIM({x_name}) != {len(shape_names)}) {{
+                PyErr_Format(PyExc_AssertionError,
+                    "SpecifyShape: Got %d dimensions, expected %d dimensions.",
+                    PyArray_NDIM({x_name}), {len(shape_names)}
+                );
+                {fail};
+            }}
+            """
+        )
 
-        raise NotImplementedError()
+        for i, (shp_name, shp) in enumerate(zip(shape_names, node.inputs[1:])):
+            if NoneConst.equals(shp):
+                continue
+            code += dedent(
+                f"""
+                if (py_{shp_name} != Py_None){{
+                    dtype_{shp_name} shp = ((dtype_{shp_name}*)PyArray_GETPTR1({shp_name}, 0))[0];
+                    if (PyArray_DIMS({x_name})[{i}] != shp) {{
+                        PyErr_Format(PyExc_AssertionError,
+                            "SpecifyShape: dim %d of input has shape %d, expected %d.",
+                            {i}, PyArray_DIMS({x_name})[{i}], shp
+                        );
+                        {fail};
+                    }}
+                }}
+                """
+            )
+
+        code += dedent(
+            f"""
+            Py_XDECREF({o_name});
+            {o_name} = {x_name};
+            Py_XINCREF({o_name});
+            """
+        )
+        return code
 
     def c_code_cache_version(self):
-        version = []
-        # If any of the c code is unversioned, we have to return ()
-        # Else, we will return a list of (type name, version) pairs.
-        for t, (c, v, _) in sorted(
-            self.c_code_and_version.items(), key=lambda pair: str(pair[0])
-        ):
-            if not v:
-                warnings.warn(
-                    "Type %s has C code for SpecifyShape, but it "
-                    "has no version. You should add a 'version' "
-                    "keyword arg when calling "
-                    "register_specify_shape_c_code." % t,
-                    stacklevel=2,
-                )
-                return ()
-            version.append((str(t), v))
-
-        return tuple(version)
+        return (2,)
 
 
-specify_shape = SpecifyShape()
+_specify_shape = SpecifyShape()
+
+
+def specify_shape(
+    x: Union[np.ndarray, Number, Variable],
+    shape: Union[
+        int, List[Union[int, Variable]], Tuple[Union[int, Variable]], Variable
+    ],
+):
+    """Specify a fixed shape for a `Variable`.
+
+    If a dimension's shape value is ``None``, the size of that dimension is not considered fixed/static at runtime.
+    """
+
+    if not isinstance(shape, (tuple, list)):
+        shape = (shape,)
+
+    # If shape is a symbolic 1d vector of fixed length, we separate the items into a
+    # tuple with one entry per shape dimension
+    if len(shape) == 1 and shape[0] is not None:
+        shape_vector = at.as_tensor_variable(shape[0])
+        if shape_vector.ndim == 1:
+            try:
+                shape = tuple(shape_vector)
+            except ValueError:
+                raise ValueError("Shape vector must have fixed dimensions")
+
+    return _specify_shape(x, *shape)
 
 
 @_get_vector_length.register(SpecifyShape)
 def _get_vector_length_SpecifyShape(op, var):
     try:
-        return aet.get_scalar_constant_value(var.owner.inputs[1])
+        return at.get_scalar_constant_value(var.owner.inputs[1]).item()
     except NotScalarConstantError:
         raise ValueError(f"Length of {var} cannot be determined")
 
@@ -530,9 +591,9 @@ class Reshape(COp):
         return f"{self.__class__.__name__}{{{self.ndim}}}"
 
     def make_node(self, x, shp):
-        x = aet.as_tensor_variable(x)
+        x = at.as_tensor_variable(x)
         shp_orig = shp
-        shp = aet.as_tensor_variable(shp, ndim=1)
+        shp = at.as_tensor_variable(shp, ndim=1)
         if not (
             shp.dtype in int_dtypes
             or (isinstance(shp, TensorConstant) and shp.data.size == 0)
@@ -552,7 +613,7 @@ class Reshape(COp):
                 shp_list = [shp_orig]
             for index in range(self.ndim):
                 y = shp_list[index]
-                y = aet.as_tensor_variable(y)
+                y = at.as_tensor_variable(y)
                 # Try to see if we can infer that y has a constant value of 1.
                 # If so, that dimension should be broadcastable.
                 try:
@@ -658,7 +719,7 @@ class Reshape(COp):
             return [
                 tuple(
                     [
-                        aet.switch(eq(requ[i], -1), rest_size, requ[i])
+                        at.switch(eq(requ[i], -1), rest_size, requ[i])
                         for i in range(self.ndim)
                     ]
                 )
@@ -701,14 +762,14 @@ class Reshape(COp):
 
 def reshape(x, newshape, ndim=None):
     if ndim is None:
-        newshape = aet.as_tensor_variable(newshape)
+        newshape = at.as_tensor_variable(newshape)
         if newshape.ndim != 1:
             raise TypeError(
                 "New shape in reshape must be a vector or a list/tuple of"
                 f" scalar. Got {newshape} after conversion to a vector."
             )
         try:
-            ndim = aet.get_vector_length(newshape)
+            ndim = get_vector_length(newshape)
         except ValueError:
             raise ValueError(
                 f"The length of the provided shape ({newshape}) cannot "
@@ -732,7 +793,7 @@ def shape_padleft(t, n_ones=1):
     Dimshuffle
 
     """
-    _t = aet.as_tensor_variable(t)
+    _t = at.as_tensor_variable(t)
 
     pattern = ["x"] * n_ones + [i for i in range(_t.type.ndim)]
     return _t.dimshuffle(pattern)
@@ -748,7 +809,7 @@ def shape_padright(t, n_ones=1):
     Dimshuffle
 
     """
-    _t = aet.as_tensor_variable(t)
+    _t = at.as_tensor_variable(t)
 
     pattern = [i for i in range(_t.type.ndim)] + ["x"] * n_ones
     return _t.dimshuffle(pattern)
@@ -776,7 +837,7 @@ def shape_padaxis(t, axis):
     Dimshuffle
 
     """
-    _t = aet.as_tensor_variable(t)
+    _t = at.as_tensor_variable(t)
 
     ndim = _t.ndim + 1
     if not -ndim <= axis < ndim:
@@ -824,34 +885,3 @@ register_shape_i_c_code(
     """,
     version=3,
 )
-
-
-register_specify_shape_c_code(
-    TensorType,
-    """
-        if (PyArray_NDIM(%(iname)s) != PyArray_DIMS(%(shape)s)[0]) {
-            PyErr_Format(PyExc_AssertionError,
-                "SpecifyShape: Got %%d dimensions, expected %%d dimensions.",
-                PyArray_NDIM(%(iname)s),
-                PyArray_DIMS(%(shape)s)[0]
-            );
-            %(fail)s;
-        }
-        for(int i = 0; i < PyArray_NDIM(%(iname)s); i++){
-            dtype_%(shape)s shp = ((dtype_%(shape)s*)PyArray_GETPTR1(%(shape)s,
-                                                                     i))[0];
-            if (PyArray_DIMS(%(iname)s)[i] != shp) {
-                PyErr_Format(PyExc_AssertionError,
-                             "SpecifyShape: dim %%d of input has shape %%d,"
-                             " expected %%d.",
-                             i, PyArray_DIMS(%(iname)s)[i],
-                             shp);
-                %(fail)s;
-            }
-        }
-        Py_XDECREF(%(oname)s);
-        %(oname)s = %(iname)s;
-        Py_XINCREF(%(oname)s);
-    """,
-    version=1,
-)
