@@ -1,6 +1,6 @@
 import logging
 import warnings
-from typing import Iterable, Optional, Union
+from typing import Iterable, Optional, Tuple, Union
 
 import numpy as np
 
@@ -8,7 +8,9 @@ import aesara
 from aesara import scalar as aes
 from aesara.configdefaults import config
 from aesara.graph.basic import Variable
-from aesara.graph.type import CType
+from aesara.graph.type import HasDataType, HasShape
+from aesara.graph.utils import MetaType
+from aesara.link.c.type import CType
 from aesara.misc.safe_asarray import _asarray
 from aesara.utils import apply_across_args
 
@@ -46,11 +48,12 @@ dtype_specs_map = {
 }
 
 
-class TensorType(CType):
+class TensorType(CType, HasDataType, HasShape):
     r"""Symbolic `Type` representing `numpy.ndarray`\s."""
 
-    __props__ = ("dtype", "shape")
+    __props__: Tuple[str, ...] = ("dtype", "shape")
 
+    dtype_specs_map = dtype_specs_map
     context_name = "cpu"
     filter_checks_isfinite = False
     """
@@ -245,7 +248,7 @@ class TensorType(CType):
         if not isinstance(other, Variable):
             # The value is not a Variable: we cast it into
             # a Constant of the appropriate Type.
-            other = self.Constant(type=self, data=other)
+            other = self.constant_type(type=self, data=other)
 
         if other.type == self:
             return other
@@ -270,7 +273,7 @@ class TensorType(CType):
 
         """
         try:
-            return dtype_specs_map[self.dtype]
+            return self.dtype_specs_map[self.dtype]
         except KeyError:
             raise TypeError(
                 f"Unsupported dtype for {self.__class__.__name__}: {self.dtype}"
@@ -612,6 +615,20 @@ class TensorType(CType):
             return ()
 
 
+class DenseTypeMeta(MetaType):
+    def __instancecheck__(self, o):
+        if type(o) == TensorType or isinstance(o, DenseTypeMeta):
+            return True
+        return False
+
+
+class DenseTensorType(TensorType, metaclass=DenseTypeMeta):
+    r"""A `Type` for dense tensors.
+
+    Instances of this class and `TensorType`\s are considered dense `Type`\s.
+    """
+
+
 def values_eq_approx(
     a, b, allow_remove_inf=False, allow_remove_nan=False, rtol=None, atol=None
 ):
